mod db;
mod serve;

use std::path::PathBuf;

use clap::{Parser, Subcommand};

/// Otter is a lightweight implementation of the Gpodder API, designed to be used for small
/// personal deployments.
#[derive(Parser)]
pub struct Cli {
    #[arg(
        long = "data",
        default_value = "./data",
        value_name = "DATA_DIR",
        env = "OTTER_DATA_DIR"
    )]
    pub data_dir: PathBuf,

    #[command(subcommand)]
    pub cmd: Command,
}

#[derive(Subcommand)]
pub enum Command {
    Serve(serve::ServeCommand),
    #[command(subcommand)]
    Db(db::DbCommand),
}

impl Cli {
    pub fn run(&self) -> u8 {
        match &self.cmd {
            Command::Serve(cmd) => cmd.run(self),
            Command::Db(cmd) => cmd.run(self),
        }
    }
}
