use clap::Args;

use crate::{db, server};

/// Run the Otter web server
#[derive(Args)]
pub struct ServeCommand {
    #[arg(
        short,
        long,
        default_value = "127.0.0.1",
        value_name = "DOMAIN",
        env = "OTTER_DOMAIN"
    )]
    domain: String,

    #[arg(
        short,
        long,
        default_value = "8080",
        value_name = "PORT",
        env = "OTTER_PORT"
    )]
    port: u16,
}

impl ServeCommand {
    pub fn run(&self, cli: &super::Cli) -> u8 {
        tracing_subscriber::fmt::init();

        tracing::info!("Initializing database and running migrations");

        let pool = db::initialize_db(cli.data_dir.join(crate::DB_FILENAME), true).unwrap();

        let ctx = server::Context {
            pool: pool.clone(),
            repo: db::SqliteRepository::from(pool.clone()),
        };
        let app = server::app(ctx);

        let rt = tokio::runtime::Builder::new_multi_thread()
            .enable_all()
            .build()
            .unwrap();

        let address = format!("{}:{}", self.domain, self.port);
        tracing::info!("Starting server on {address}");

        rt.block_on(async {
            let listener = tokio::net::TcpListener::bind(address).await.unwrap();
            axum::serve(listener, app.into_make_service())
                .await
                .unwrap()
        });

        0
    }
}
